@@ -15,6 +15,7 @@
 .onAttach <- function(libname, pkgname) {
     options(enrichR.base.address = "https://maayanlab.cloud/Enrichr/")
     options(enrichR.live = TRUE)
+    options(enrichR.quiet = FALSE)
     packageStartupMessage("Welcome to enrichR\nChecking connection ... ", appendLF = TRUE)
     options(modEnrichR.use = TRUE)
     options(enrichR.sites.base.address = "https://maayanlab.cloud/")
@@ -27,7 +28,10 @@
             packageStartupMessage("Enrichr ... ", appendLF = FALSE)
             if (getOption("enrichR.live")) packageStartupMessage("Connection is Live!")
         }
-    } else {packageStartupMessage("No internet connection could be found.")}
+    } else {
+        packageStartupMessage("No internet connection could be found.")
+        options(enrichR.live = FALSE)
+    }
 }
 
 
@@ -39,18 +43,34 @@
 ##' @param ... (Optional). Additional parameters to pass to GET
 ##' @return same as GET
 ##' @author Wajid Jawaid \email{wj241@alumni.cam.ac.uk}
+##' @author I-Hsuan Lin \email{i-hsuan.lin@manchester.ac.uk}
 ##' @importFrom httr GET
+##' @importFrom httr status_code
+##' @importFrom httr http_status
 getEnrichr <- function(url, ...) {
+    options(enrichR.live = FALSE)
     tryCatch({
-        options(enrichR.live = TRUE)
-        x <- GET(url=url, ...)
+        x <- GET(url = url, ...)
+        code <- status_code(x)
+        if(code != 200) {
+            # Error with status code
+            message(http_status(code)$message)
+        } else {
+            # OK/success
+            options(enrichR.live = TRUE)
+            invisible(x)
+        }
     },
-    error = function(err_msg) {
-        message("EnrichR website not responding")
-        options(enrichR.live = FALSE)
+    # Warning message
+    warning = function(warn) {
+        message(warn); message("") # force newline
+    },
+    # Error without status code
+    error = function(err) {
+        message(err); message("") # force newline
     },
     finally = function() {
-       invisible(x) 
+        invisible(x)
     })
 }
 
@@ -69,8 +89,7 @@ listEnrichrSites <- function(...) {
         if (paste0(getOption("enrichR.sites.base.address"), site, "/")  == getOption("enrichR.base.address")) {
             if (getOption("enrichR.live")) packageStartupMessage("Connection is Live!")
         } else 
-            if (getOption("enrichR.live")) packageStartupMessage("Connection is available!")
-        
+            if (getOption("enrichR.live")) packageStartupMessage("Connection is Live!")
     }
 }
 
@@ -84,7 +103,7 @@ listEnrichrSites <- function(...) {
 ##' @export
 setEnrichrSite <- function(site) {
     site <- gsub(getOption("enrichR.sites.base.address"), "", site)
-    matched <- grep(paste0("^",site), 
+    matched <- grep(paste0("^",site),
                     getOption("enrichR.sites"),
                     ignore.case = TRUE,value = FALSE)
     if( length(matched) == 0 ) {
@@ -93,7 +112,7 @@ setEnrichrSite <- function(site) {
                 paste("-",getOption("enrichR.sites"),"\n"))
     } else if (length(matched) > 1) {
         message("Given website matches multiple options: ", site)
-        message(paste("-", getOption("enrichR.sites")[matched],"\n"),)        
+        message(paste("-", getOption("enrichR.sites")[matched],"\n"),)
     } else {
         site <- getOption("enrichR.sites")[matched]
         options(enrichR.base.address = paste0(getOption("enrichR.sites.base.address"),site,"/"))
@@ -115,7 +134,7 @@ setEnrichrSite <- function(site) {
 ##' @examples
 ##' dbs <- listEnrichrDbs()
 listEnrichrDbs <- function() {
-    dfSAF <- getOption("stringsAsFactors")
+    dfSAF <- getOption("stringsAsFactors", FALSE)
     options(stringsAsFactors = FALSE)
     dbs <- getEnrichr(url = paste0(getOption("enrichR.base.address"), "datasetStatistics"))
     if (!getOption("enrichR.live")) return()
@@ -149,7 +168,7 @@ listEnrichrDbs <- function() {
 ##' @export
 ##' @examples
 ##' dbs <- listEnrichrDbs()
-##' dbs <- c("GO_Molecular_Function_2018", "GO_Cellular_Component_2018", 
+##' dbs <- c("GO_Molecular_Function_2018", "GO_Cellular_Component_2018",
 ##'          "GO_Biological_Process_2018")
 ##' enriched <- enrichr(c("Runx1", "Gfi1", "Gfi1b", "Spi1", "Gata1", "Kdr"), dbs)
 enrichr <- function(genes, databases = NULL) {
@@ -177,7 +196,7 @@ enrichr <- function(genes, databases = NULL) {
         message("No databases have been provided")
         return()
     }
-    cat("Uploading data to Enrichr... ")
+    if (!getOption("enrichR.quiet")) cat("Uploading data to Enrichr... ")
     if (is.vector(genes) & ! all(genes == "") & length(genes) != 0) {
         temp <- POST(url=paste0(getOption("enrichR.base.address"), "enrich"),
                      body=list(list=paste(genes, collapse="\n")))
@@ -189,12 +208,12 @@ enrichr <- function(genes, databases = NULL) {
         warning("genes must be a non-empty vector of gene names or a data.frame with genes and score.")
     }
     getEnrichr(url=paste0(getOption("enrichR.base.address"), "share"))
-    cat("Done.\n")
+    if (!getOption("enrichR.quiet")) cat("Done.\n")
     dbs <- as.list(databases)
-    dfSAF <- getOption("stringsAsFactors")
+    dfSAF <- getOption("stringsAsFactors", FALSE)
     options(stringsAsFactors = FALSE)
     result <- lapply(dbs, function(x) {
-        cat("  Querying ", x, "... ", sep="")
+      if (!getOption("enrichR.quiet")) cat("  Querying ", x, "... ", sep="")
         r <- getEnrichr(url=paste0(getOption("enrichR.base.address"), "export"),
                         query=list(file="API", backgroundType=x))
         if (!getOption("enrichR.live")) return()
@@ -202,13 +221,13 @@ enrichr <- function(genes, databases = NULL) {
         tc <- textConnection(r)
         r <- read.table(tc, sep = "\t", header = TRUE, quote = "", comment.char="")
         close(tc)
-        cat("Done.\n")
+        if (!getOption("enrichR.quiet")) cat("Done.\n")
         return(r)
     })
     options(stringsAsFactors = dfSAF)
-    cat("Parsing results... ")
+    if (!getOption("enrichR.quiet")) cat("Parsing results... ")
     names(result) <- dbs
-    cat("Done.\n")
+    if (!getOption("enrichR.quiet")) cat("Done.\n")
     return(result)
 }
 
@@ -244,39 +263,49 @@ enrichr <- function(genes, databases = NULL) {
     return(df)
 }
 
-##' Print Enrichr output.
+##' Print Enrichr results
 ##'
-##' Print Enrichr output to text file.
+##' Print Enrichr results from the selected gene-set libraries to individual text files.
 ##' @title printEnrich
-##' @param data (Required). Output from Enrichr function.
+##' @param data (Required). Output list object from the \code{"enrichr"} function.
 ##' @param prefix (Optional). Prefix of output file. Default is \code{"enrichr"}.
-##' @param showTerms (Optional). Number of terms to show. 
+##' @param showTerms (Optional). Number of terms to show.
 ##' Default is \code{NULL} to print all terms.
-##' @param columns (Optional). Columns from each entry of data. 
+##' @param columns (Optional). Columns from each entry of data.
 ##' Default is \code{c(1:9)} to print all columns.
-##' 1-"Term", 2-"Overlap", 3-"P.value", 4-"Adjusted.P.value" 5-"Old.P.value", 
+##' 1-"Term", 2-"Overlap", 3-"P.value", 4-"Adjusted.P.value" 5-"Old.P.value",
 ##' 6-"Old.Adjusted.P.value" 7-"Odds.Ratio" 8-"Combined.Score" 9-"Combined.Score"
+##' @param write2file (Optional). Set to TRUE if you would like this functino to
+##' output a file
+##' @param outFile (Optional). Output file format, choose from "txt" and "excel". Default is "txt".
 ##' @return NULL
 ##' @author Wajid Jawaid \email{wj241@alumni.cam.ac.uk}
 ##' @author I-Hsuan Lin \email{i-hsuan.lin@manchester.ac.uk}
 ##' @importFrom utils write.table
+##' @importFrom WriteXLS WriteXLS
 ##' @export
 ##' @examples
 ##' if (getOption("enrichR.live")) {
 ##'   enrichRLive <- TRUE
 ##'   dbs <- listEnrichrDbs()
 ##'   if(is.null(dbs)) enrichRLive <- FALSE
-##'   dbs <- c("GO_Molecular_Function_2018", "GO_Cellular_Component_2018", 
+##'   dbs <- c("GO_Molecular_Function_2018", "GO_Cellular_Component_2018",
 ##'            "GO_Biological_Process_2018")
 ##'   enriched <- enrichr(c("Runx1", "Gfi1", "Gfi1b", "Spi1", "Gata1", "Kdr"), dbs)
-##'   if (enrichRLive) printEnrich(enriched)
+##'   if (enrichRLive) printEnrich(enriched, write2file = FALSE)
 ##' }
-printEnrich <- function(data, prefix = "enrichr", showTerms = NULL, columns = c(1:9)) {
+printEnrich <- function(data, prefix = "enrichr", showTerms = NULL, columns = c(1:9),
+                        write2file = TRUE, outFile = c("txt", "excel")) {
     if (!is.list(data)) stop("data is malformed must be a list")
-    if(!is.numeric(columns)) {
+    if (!is.numeric(columns)) {
         stop(paste0("columns '", columns, "' is invalid."))
     }
-
+		outFile <- match.arg(outFile)
+		if (outFile == "excel") {
+			filename <- paste0(prefix, ".xlsx")
+			output_excel_df = vector("list", length(data))
+		}
+		
     for (i in 1:length(data)) {
         dbname <- names(data)[i]
         df <- data[[i]]
@@ -288,12 +317,23 @@ printEnrich <- function(data, prefix = "enrichr", showTerms = NULL, columns = c(
         if(any(columns > ncol(df))) {
             stop("Undefined columns selected")
         }
+        if (write2file) {
+            if (outFile == "txt") {
+                filename <- paste0(prefix, "_", dbname, ".txt")
+                write.table(df, file = filename, sep = "\t", quote = F, row.names = F, col.names = T)
+            } else {
+                output_excel_df[[i]] <- df
+                if (i == length(data)) {
+                    WriteXLS(output_excel_df, ExcelFileName = filename, SheetNames = names(data))
+                }
+            }
+        }
     }
 }
 
 ##' Visualise a Enrichr output as barplot
 ##'
-##' Print Enrichr output to text file.
+##' Visualise Enrichr result from a selected gene-set library as barplot.
 ##' @title plotEnrich
 ##' @param df (Required). A single data.frame from a list of Enrichr output.
 ##' @param showTerms (Optional). Number of terms to show. Default is \code{20}.
@@ -321,6 +361,7 @@ printEnrich <- function(data, prefix = "enrichr", showTerms = NULL, columns = c(
 ##' @importFrom ggplot2 coord_flip
 ##' @importFrom ggplot2 theme_bw
 ##' @importFrom ggplot2 scale_fill_continuous
+##' @importFrom ggplot2 scale_x_discrete
 ##' @importFrom ggplot2 guides
 ##' @importFrom ggplot2 guide_colorbar
 ##' @importFrom ggplot2 theme
@@ -335,10 +376,10 @@ printEnrich <- function(data, prefix = "enrichr", showTerms = NULL, columns = c(
 ##'   dbs <- listEnrichrDbs()
 ##'   enrichRLive <- TRUE
 ##'   if (is.null(dbs)) enrichRLive <- FALSE
-##'   dbs <- c("GO_Molecular_Function_2018", "GO_Cellular_Component_2018", 
+##'   dbs <- c("GO_Molecular_Function_2018", "GO_Cellular_Component_2018",
 ##'            "GO_Biological_Process_2018")
 ##'   enriched <- enrichr(c("Runx1", "Gfi1", "Gfi1b", "Spi1", "Gata1", "Kdr"), dbs)
-##'   # Plot top 20 GO-BP results ordered by P-value
+##'   # Plot top 20 terms from "GO_Biological_Process_2018" and ordered by P-value
 ##'   if (enrichRLive) {
 ##'     plotEnrich(enriched[[3]], showTerms = 20, numChar = 50, y = "Count",
 ##'                orderBy = "P.value")
@@ -346,8 +387,12 @@ printEnrich <- function(data, prefix = "enrichr", showTerms = NULL, columns = c(
 ##' }
 plotEnrich <- function(df, showTerms = 20, numChar = 40, y = "Count", orderBy = "P.value",
                        xlab = NULL, ylab = NULL, title = NULL) {
-    if(!is.data.frame(df)) {stop("df is malformed - must be a data frame")}
-    if(nrow(df) == 0 | ncol(df) == 0) {stop("df is malformed - must be a data frame")}
+    if(!is.data.frame(df)) {
+        stop("Input df is malformed - must be a data.frame object.")
+    }
+    if(nrow(df) == 0 | ncol(df) == 0) {
+        stop("Input df is empty.")
+    }
     if(!is.numeric(numChar)) {
         stop(paste0("numChar '", numChar, "' is invalid."))
     }
@@ -357,8 +402,13 @@ plotEnrich <- function(df, showTerms = 20, numChar = 40, y = "Count", orderBy =
     # Create trimmed name (as seen in topGO)
     shortName <- paste(substr(df$Term, 1, numChar),
                        ifelse(nchar(df$Term) > numChar, '...', ''), sep = '')
-    df$shortName = shortName
-    df$shortName <- factor(df$shortName, levels = rev(unique(df$shortName)))
+    names(shortName) <- df$Term
+
+    # Print warning if there are any duplicated trimmed names
+    if(any(duplicated(shortName))) {
+        warning("There are duplicated trimmed names in the plot, consider increasing the 'numChar' setting.")
+    }
+
     df$Ratio <- df$Significant/df$Annotated
 
     # Define fill variable (P.value or Combined.Score)
@@ -374,7 +424,7 @@ plotEnrich <- function(df, showTerms = 20, numChar = 40, y = "Count", orderBy =
     }
 
     # Define variable mapping
-    map <- aes_string(x = "shortName", y = y, fill = fill)
+    map <- aes_string(x = "Term", y = y, fill = fill)
 
     # Define labels
     if(is.null(xlab)) {
@@ -394,7 +444,8 @@ plotEnrich <- function(df, showTerms = 20, numChar = 40, y = "Count", orderBy =
     }
 
     # Make the ggplot
-    p <- ggplot(df, map) + geom_bar(stat = "identity") + coord_flip() + theme_bw()
+    p <- ggplot(df, map) + geom_bar(stat = "identity") + coord_flip() + theme_bw() +
+        scale_x_discrete(labels = rev(shortName), limits = rev(df$Term))
 
     if(orderBy == "Combined.Score") {
         p <- p + scale_fill_continuous(low = "blue", high = "red") +
@@ -414,4 +465,3 @@ plotEnrich <- function(df, showTerms = 20, numChar = 40, y = "Count", orderBy =
 
     return(p)
 }
-
